
#-----------------------------------------------------------------------------
# Name        	: SkIndigo_07.rb
# Description 	: Model exporter and material editor for Indigo 0.7test5
# Menu Item   	: Plugins\SkIndigo\
# Author      	: Dale Martens (Whaat)
#		  Code used from following SU exporters: Su2KT by Tomasz, SU2POV by Didier Bur and OGRE exporter by Kojack
# Usage     	: Copy script to PLUGINS folder in SketchUp folder, run SU, go to Plugins\SkIndigo
# Date        	: 4/7/2007
# Type        	: Exporter
# Version     	: 0.7test5

#Version 0.7test5

#Added: Environment Map Support
#Added: Up to 4 threads selectable
#Changed: Mirror preset is now a specular material with high IOR (perfect dielectric)
#Added: Support for most features of Indigo 0.7
#Changed: Removed tonemapping from render settings - use Indigo tonemapper GUI
#Added: Bump map support to metals (complex IOR)
#Changed: Changed default exponent to 1000
#Added: Material editor context menu
#Added: New sss material and light presets
#Changed: aperture setting is now specified as f-stop
#Changed: minor UI changes
#Added: autofocus is selectable through focus tool
#Added: reverse gamma correction to rgb meshlights
#Added: context menu to apply material presets and quick export
#Added: New Render Modes - Export All, Separate Settings, and Settings Only (Quick Export)
#Added: Option to run Indigo in Low Priority (creates batch file) - May only work with PCs
#Changed: Material Table to Query Material Types
#Changed: Only visible geometry is exported
#Changed: Auto-render only allowed when file is exported to Indigo directory
#Changed: BPT always turned off (not working in Indigo 0.7test5)
#Changed: Default MNCR to 500 from 100.  500 works better for SSS


#Version 0.6 Official

#Changes since Beta1 - Initial Test Release
#Fixed: [,,,,] characters corrupting inifile.txt on some sytems
#Changed: Auto-render now aborts if there are no lights in scene
#Changed: Changed the way NK_Data (Complex IOR) materials are selected
#Fixed: imported 3DS materials automatically being assigned Specular material type
#Fixed: applied reverse gamma correction exponent of 2.3 to diffuse color of materials
#Changed: Sketchup camera aspect ratio will change based on width and height of render.	
#Changed: Focal Length in Set Focus Tool changed to Focus Distance
#Added: Monochrome Render Modes
#Added: Dull plastic, shiny plastic, leather, car paint, complex IOR presets
#Changed: Specular Materials can now be transparent or non-transparent depending on the opacity setting
#Changed: Negative bump values are now allowed
#Changed: Changed default fresnel to 1.0
#-----------------------------------------------------------------------------

require 'sketchup.rb'
		

class SU2IN

# ---- Settings and global variables ----- #

def SU2IN::reset_global_variables

model = Sketchup.active_model

	$n_meshlights=0	
	$sun=false
	$n_cameras=0
	$face=0
	$g_su2in_scale = 0.0254
	$g_su2in_copy_textures = true
	$g_su2in_export_materials = true
	$g_su2in_export_meshes = true
	$g_su2in_export_lights = true
	$g_su2in_model_name="" if not $g_su2in_model_name
	$g_su2in_textures_prefix = "TX_"
	$g_su2in_model_textures=[]
	$g_su2in_count_tri = 0
	$g_su2in_count_material = 0
	$g_su2in_faces = []
	$g_su2in_trans = []
	$g_su2in_lights = []
	$animation=false
	$g_su2in_materials = {}
	$g_su2in_fm_materials = {}
	$g_su2in_selected=false	
	$g_su2in_clay=false
	$monochrome=false
	$parent_mat=[]
	$fm_comp=[]
	$current_camera=model.active_view.camera
	$current_camera.image_width=35.0
	$selected_material=nil if not $selected_material
	$default_material_exported=false
	$monochrome_material=""
	$null_exported=false
	$mode="All" if not $mode
	$mat_path="" if not $mat_path
	$ss_exported=false if not $ss_exported
	$settings_only=false if not $settings_only

#Render Settings - Modify these values to change defaults

	
$width=(model.get_attribute 'indigo07_settings','w',"800").to_i
$height=(model.get_attribute 'indigo07_settings','h',"600").to_i
$aperture=(model.get_attribute 'indigo07_settings','ap',"f\/22").to_s
$white_balance=model.get_attribute 'indigo07_settings','wb',"E"
$environment=model.get_attribute 'indigo07_settings','env',"Sun and Sky"
$turbidity=(model.get_attribute 'indigo07_settings','tur',"2.0").to_f
$bidir=model.get_attribute 'indigo07_settings','bidir',"Off"
$mncr=(model.get_attribute 'indigo07_settings','mncr',"500").to_i
$image_save_period=(model.get_attribute 'indigo07_settings','isp',"20").to_i
$halt_time=(model.get_attribute 'indigo07_settings','halt',"-1").to_i
$threads=(model.get_attribute 'indigo07_settings','threads',"1").to_s
$focal_point=(model.get_attribute 'indigo07_settings','fp',"auto")
$auto_render=model.get_attribute 'indigo07_settings','auto',"On"
$indigo_path=model.get_attribute 'indigo07_settings','indigo_path',""
$mode=model.get_attribute 'indigo07_settings','mode',"Separate Settings"
	
$exr=model.get_attribute 'indigo07_settings','exr',""
$exr_type=model.get_attribute 'indigo07_settings','exr_type',"Lat_Long"
$exr_width=(model.get_attribute 'indigo07_settings','exr_width',"640").to_i
$exr_gain=(model.get_attribute 'indigo07_settings','exr_gain',"1.0").to_f
	
 
end


##### ------  Exporting starts here ------ ######

def SU2IN::export

	
	SU2IN.reset_global_variables
	$mode="Settings Only" if $settings_only

	model = Sketchup.active_model
	
	if $mode!="Settings Only"

		$g_su2in_model_name=""
		model_filename = File.basename(model.path)
			if model_filename!=""
				model_name = model_filename.split(".")[0]
				model_name += ".xml"
			else
				model_name = "Untitled.xml" 	
			end
		
		$g_su2in_export_file=UI.savepanel("Export Model", "" , model_name)

		return if $g_su2in_export_file==nil		

			if $g_su2in_export_file==$g_su2in_export_file.split(".")[0] #no extension given
				$g_su2in_export_file+=".xml"
			end
		$g_su2in_model_name=File.basename($g_su2in_export_file)
		
		
		
		$mat_path=$g_su2in_export_file.split($g_su2in_model_name)[0] + $g_su2in_model_name.split(".")[0] + "-settings.xml"
		
		$g_su2in_model_name=$g_su2in_model_name.split(".")[0]
	end

	renderingoptions=model.rendering_options
	mode=renderingoptions["RenderMode"]
	
	
	if mode==5 and $mode!="Settings Only"
		materials = model.materials
    		translatednames = materials.collect {|m| m.display_name}
		names = materials.collect {|m| m.name}
		dropdowns=["Clay","Default"]
		dropdowns=dropdowns.push(translatednames)
		prompts=["Choose Material"]
		values=["Clay"]
		enums=[dropdowns.join("|")]
		results=inputbox prompts,values, enums,"Monochrome Render"
		return if not results
		if results[0]=="Clay"
			$g_su2in_clay=true
			$monochrome_material="Default"
		else
			$monochrome=true
			mat=results[0]
			$monochrome_material=mat.gsub(/[<>]/, '*')
					
		end
	end


	if $g_su2in_model_name!=""	

		if ((model.get_attribute 'indigo07_settings','auto',"On")=="On") and ($mode !="Settings Only")
			SU2IN::modify_indigo_inifile 
		end
		
		out = File.new($g_su2in_export_file,"w") if $mode!="Settings Only"
		out_mat=File.new($mat_path,"w") if $mode!="All"

		$g_su2in_path_textures=File.dirname($g_su2in_export_file)

		start_time=Time.new

			
		

		if $mode=="All"

			out.puts "<?xml version=\"1.0\" standalone=no>"
			out.puts "<scene>"

			SU2IN.export_global_settings(out)

			SU2IN.param_cam_view(model.active_view, out)
		

			SU2IN.write_sky(out)

			materials = model.materials
			blends=[]
	
			for mat in materials
				if (mat.get_attribute 'indigo07_materials','type',"Diffuse")=="Blend"
					blends.push(mat)
				else
					SU2IN.export_mat(out,mat)
				end
			end

			for mat in blends
				SU2IN.export_mat(out,mat)
			end

			SU2IN.export_mesh(out)
			out.puts "</scene>"
			out.close
			$ss_exported=false
		
		else
			
			if $mode=="Separate Settings"
				out.puts "<?xml version=\"1.0\" standalone=no>"
				out.puts "<scene>"
				out.puts "		<include>"
				out.puts "		<pathname>#{File.basename($mat_path)}</pathname>"
				out.puts "		</include>"
				SU2IN.export_mesh(out)
				out.puts "</scene>"
				out.close
				
			end

			out_mat.puts "<scenedata>"

			SU2IN.export_global_settings(out_mat)
			SU2IN.param_cam_view(model.active_view, out_mat)
			SU2IN.write_sky(out_mat)

			materials = model.materials
			blends=[]
	
			for mat in materials
				if (mat.get_attribute 'indigo07_materials','type',"Diffuse")=="Blend"
					blends.push(mat)
				else
					SU2IN.export_mat(out_mat,mat)
				end
			end

			for mat in blends
				SU2IN.export_mat(out_mat,mat)
			end

			out_mat.puts "</scenedata>"
			out_mat.close
			$ss_exported=true
			
		end	
		

	if ($g_su2in_copy_textures == true and $g_su2in_model_textures!=[])

		su2in_texturewriter = Sketchup.create_texture_writer
		SU2IN.status_bar("Exporting textures")		

		if FileTest.exist? ($g_su2in_path_textures+"\\"+$g_su2in_textures_prefix+$g_su2in_model_name)
		else
			Dir.mkdir($g_su2in_path_textures+"\\"+$g_su2in_textures_prefix+$g_su2in_model_name)
		end

		for i in $g_su2in_model_textures
			su2in_texturewriter.load(i[1],true)
		end

		
		status=su2in_texturewriter.write_all $g_su2in_path_textures+"\\"+$g_su2in_textures_prefix+$g_su2in_model_name,false
		
		if status
			stext = "All textures and model"
		else
			stext = "An error occured when exporting textures. Model"		
		end
		
		su2in_texturewriter=nil

	else
		stext = "Model"
	end

		
		end_time=Time.new
		elapsed=end_time-start_time
		time=" exported in "
			(time=time+"#{(elapsed/3600).floor}h ";elapsed-=(elapsed/3600).floor*3600) if (elapsed/3600).floor>0
			(time=time+"#{(elapsed/60).floor}m ";elapsed-=(elapsed/60).floor*60) if (elapsed/60).floor>0
			time=time+"#{elapsed.round}s. "

		SU2IN.status_bar(stext+time+" Triangles = #{$g_su2in_count_tri}")
		export_text="Model saved in file:\n"
		
		(UI.messagebox export_text + $g_su2in_export_file) if $auto_render=="Off"

	end

	

		
#--------------Open Indigo for rendering
	if ($auto_render=="On") and ($indigo_path !="")
		if ($n_meshlights==0) and ($sun==false) and ($environment!="Background") and ($environment!="Environment Map") and ($mode!="Settings Only")
			UI.messagebox "Warning: There are no lights in your scene.  Auto-render aborted."
			UI.messagebox export_text + $g_su2in_export_file
			SU2IN.reset_global_variables
			return
		end

		if $threads=="Low Priority"
			command_line=create_batch_file($indigo_path)
			command_line=command_line.gsub("\\","\/")
		else
			command_line=$indigo_path.gsub("\\","\/")
		end

		if File.dirname($indigo_path)==File.dirname($g_su2in_export_file)
			UI.openURL(command_line)
		else
			UI.messagebox ("To auto-render from Sketchup, you must export the model to the Indigo directory. Open \"indigo.exe\" to render.")
		end
		SU2IN.reset_global_variables
	else
		SU2IN.reset_global_variables
		
	end
	$settings_only=false
	
end


# ----------------------------------- Render Settings

def SU2IN::render_settings

SU2IN.reset_global_variables

#Get Current Settings

model=Sketchup.active_model

prompts=["Width","Height","Aperture (f-stop)","White Balance","Environment","Sky Turbidity (>0)","Bidirectional Path Tracing","Max Num Consec Rej (>0)","Image Save Period (s)","Halt Time (s)","Number of Threads","Auto-Render after Export","Export Mode"]
values=[$width,$height,$aperture,$white_balance,$environment,$turbidity,$bidir,$mncr,$image_save_period,$halt_time,$threads,$auto_render,$mode]
wb_choices=%w[E D50 D55 D65 D75 A B C 9300 F2 F7 F11].join("|")
env_choices=["Sun and Sky","Background","Environment Map"].join("|")
ap_choices=["f\/1.0","f\/1.4","f\/2","f\/2.8","f\/4","f\/5.6","f\/8","f\/11","f\/16","f\/22","f\/32","f\/45","f\/64","f\/90","f\/128"]
bi_choices=%w[On Off].join("|")
th_choices=["Low Priority","1","2","3","4"].join("|")
auto_choices=%w[On Off].join("|")
mode_choices=["All","Separate Settings"].join("|")

dropdowns = [nil,nil,ap_choices.join("|"),wb_choices,env_choices,nil,bi_choices,nil,nil,nil,th_choices,auto_choices,mode_choices] 

results=inputbox prompts,values,dropdowns,"Indigo Render Settings"
return nil if not results

#DATA Validation

results[8]=-1 if results[8] <-1


#------------------------store settings in attribute dictionary and save to global variables

$width=model.set_attribute 'indigo07_settings','w',results[0].abs 
$height=model.set_attribute 'indigo07_settings','h',results[1].abs
$aperture=model.set_attribute 'indigo07_settings','ap',results[2]
$white_balance=model.set_attribute 'indigo07_settings','wb',results[3]
$environment=model.set_attribute 'indigo07_settings','env',results[4]
$turbidity=model.set_attribute 'indigo07_settings','tur',results[5].abs
$bidir=model.set_attribute 'indigo07_settings','bidir',results[6]
$mncr=model.set_attribute 'indigo07_settings','mncr',results[7].abs
$image_save_period=model.set_attribute 'indigo07_settings','isp',results[8].abs
$halt_time=model.set_attribute 'indigo07_settings','halt',results[9]
$threads=model.set_attribute 'indigo07_settings','threads',results[10]
$auto_render=model.set_attribute 'indigo07_settings','auto',results[11]
$mode=model.set_attribute 'indigo07_settings','mode',results[12]

$current_camera.aspect_ratio=($width.to_f)/($height.to_f)


end

# ----------------------------------- About message
def SU2IN::about
UI.messagebox("SkIndigo 0.7test5 Released 7 April 2007
Freeware SketchUp Exporter to Indigo 0.7test5
Author: Whaat\n\nBased on SU2KT written by Tomasz
SU2POV written by Didier Bur
and OGRE exporter by Kojack

Please see Indigo's website if you have questions about Indigo
http://www.indigorenderer.com

HOW TO:

Render with Sun and Sky Light
1. Turn on shadows
2. Turn on Sun and Sky in SkIndigo Render Settings

Render with Custom Lights
1. Use either an RGB Light or a Blackbody Light material in your scene.
2. RGB lights will emit light according to the material color
3. Blackbody lights will emit light according to the temperature
4. Light is only emitted from the front side of the face

Render with Uniform Background Light
1. Turn on Background in SkIndigo Render Settings
2. Scene will be lit with a uniform background light matching the current Sketchup background color

Render with Environment Map
1. Turn on Environment Map in SkIndigo Render Settings
2. Set your Environment Map using the SkIndigo menu item
3. You can use EXR maps in Lat_Long format or FLOAT maps in Spherical format

Monochrome Render (Clay)
1. Change Sketchup render mode to monochrome
2. Choose Clay to render clay mode or any other material
3. Choose Default to render using the current default material color

Adjust the Rendered Image Tones
1. Make sure you have downloaded the Indigo Tonemapper application.
2. Load the appropriate .igi file using the Indigo Tonemapper application.

Create Transparent Material
1. Enable transparency in Sketchup material settings using the opacity slider
2. Set the material type to SPECULAR
3. The opacity slider will change the amount of RGB absorptivity. Set to 0 for no absorptivity (i.e. clear glass).
4. SPECULAR materials should have thickness to render properly.

Create Material with Specular Highlights 
1. Set the material type to Phong
2. Set the index of refraction to >1.0
3. Use the exponent value to control the glossiness.

Create Diffuse (Lambertian) Material
1. Set the material type to Diffuse

Use BLEND materials
1. Set the material type to BLEND
2. If a texture is applied to the material, it will be used as the clipping map
3. If no texture is applied, the blend factor will control the weight of the materials

Use Sub-surface scattering
1. Enable transparency in Sketchup material settings using the opacity slider
2. Set the material type to SPECULAR
3. The opacity slider will change the amount of RGB absorptivity. Set to 0 for no absorptivity
4. Enable SSS with the material editor
5. Input the Uniform scattering coefficient or the 3 RGB scattering coefficients separated by spaces
4. SPECULAR materials should have thickness to render properly.

Use Material Presets
1. Presets are intended to give you approximate starting values that you can further refine to suit your needs
2. These values will be updated in future releases as the Indigo material library continues to be developed

Use Textures
1. Apply textures directly to individual faces, not groups or components
2. Currently, textures distorted with the position texture tool are not supported
3. JPEG and PNG textures are supported but PNG textures may cause Indigo to crash

Use Bump and Exponent Maps
1. You can select any texture that is currently in your model
2. The map will be positioned using the same coordinates as the currently applied color texture.  If no color texture is applied, the bump map will not render correctly
3. If you wish to use a bump map different then the color texture,they should be the same size or you will get strange results.
4. Your maps will have to be manually exported to the TX_EXPORTNAME directory if they are not currently applied to a visible face in your model. Right-click on material in Paint Bucket
5. Set bump or exponent gain

Resume a Render from an .igi file
1. From the command line type \"indigo.exe -r renders/your_igi.igi your_xml.xml\"
2. In this case, the .igi file is located in the indigo/renders directory and XML input file is located in the indigo directory.

TIPS:

1. Use Sketchup shortcut keys!  Especially for assigning properties to materials
2. Be sure that no faces are reversed.  Use monochrome mode to check for reversed faces
3. Use layers to separate geometry and lights.  For example, put different lighting set-ups on different layers.  If you are doing material tests, put unnecessary objects on a hidden layer
4. Always export your model to the Indigo directory.  This allows use of Auto-render
5. If you are getting strange shadows in your render, try disabling smoothing.  Also, check orientation of faces
6. Modify the aperture size to control depth of field
7. If your image is blurry, use the Set Camera Focus Tool

Indigo 0.7 features currently not supported:
1. Peak spectra
2. Henyey Greenstein phase function
3. Dermis and epidermis media
4. a and c texture coefficients
5. Comprehensive nkdata list
6. Instancing
7. Primitives and rectanglelight

KNOWN ISSUES/BUGS:
1. Distorted textures not yet supported
2. Textures applied to groups and components not yet supported
3. Quick Export and Low Priority modes are experimental and may be prone to bugs.


" , MB_MULTILINE , "About SkIndigo 0.7test5 - Exporter to Indigo 0.7test5") 

end

# -----------Extract the camera parameters of the current view ------------------------------------
def SU2IN::param_cam_view( v , out)

h = v.vpheight
w = v.vpwidth
user_camera = v.camera
user_pers = user_camera.perspective?
user_eye = user_camera.eye

user_x = user_camera.xaxis 
user_y = user_camera.yaxis 
user_z = user_camera.zaxis


	if( user_pers == true ) 
  	user_foc = user_camera.focal_length

	else 
		user_foc = 36

	end
	
	w_x_h = w.to_s + "x" + h.to_s
	w_x_h=$resolution if $animation==true
	SU2IN.write_camera("\#\# Current View \#\#", user_eye, user_x, user_y, user_z, w_x_h, user_foc, out)

end


#---------------- Save camera to file ------------------------#

def SU2IN::write_camera( p, user_eye, user_x, user_y, user_z, w_x_h, user_foc, out)


cx=user_eye.x.to_m
cy=user_eye.y.to_m
cz=user_eye.z.to_m

#display warning message if camera is outside the sky sphere
origin = Geom::Point3d.new 0,0,0
camera_distance = origin.distance user_eye
if (camera_distance>11800) and ($environment!="Environment Map")
	UI.messagebox "Warning: Camera is outside the sky sphere."
end


rvx=user_x.x.to_cm
rvy=user_x.y.to_cm
rvz=user_x.z.to_cm

uvx=user_y.x.to_cm
uvy=user_y.y.to_cm
uvz=user_y.z.to_cm

fvx=user_z.x.to_cm
fvy=user_z.y.to_cm
fvz=user_z.z.to_cm

aspect=$width.to_f/$height.to_f
sensor_distance=$current_camera.focal_length/1000
sensor_width=$current_camera.image_width/1000

stop=$aperture.split("\/")[1].to_f
ap_radius=sensor_distance/(stop*2)

if $focal_point!="auto"
	vector=user_eye - $focal_point if $focal_point
	$focus_distance=vector.length.to_m
end


out.puts "		<camera>"
out.puts "			<pos>" + "%.4f" %(cx)+ " "+ "%.4f" % (cy) + " " + "%.4f" %(cz)+ "</pos>"
out.puts "			<up>" + "%.4f" %(uvx)+ " "+ "%.4f" % (uvy) + " " + "%.4f" %(uvz)+ "</up>"
out.puts "			<forwards>" + "%.4f" %(fvx)+ " "+ "%.4f" % (fvy) + " " + "%.4f" %(fvz)+ "</forwards>"
out.puts "			<aperture_radius>#{ap_radius}</aperture_radius>"
out.puts "			<focus_distance>#{$focus_distance}</focus_distance>"
out.puts "			<aspect_ratio>" + "%.4f" %(aspect) + "</aspect_ratio>"
out.puts "			<sensor_width>#{sensor_width}</sensor_width>"
out.puts "			<lens_sensor_dist>#{sensor_distance}</lens_sensor_dist>"
out.puts "			<white_balance>#{$white_balance}</white_balance>"
out.puts "			<autofocus/>" if $focal_point=="auto"
out.puts "		</camera>"


end

#method to get an objects color

def SU2IN::get_object_color (obj)
  if (obj.material == nil)
    col = Sketchup::Color.new [255,255,255]
    else
    col = obj.material.color
  end
return col
end


# -------------------------------- Export Physical Sky and Background settings

def SU2IN::write_sky ( out )
	si = Sketchup.active_model.shadow_info
	
	if si["DisplayShadows"]==true	#DisplayShadows True or False
		$sun=true
	else
		$sun=false
	end
	
	
	v = si["SunDirection"]

	point = Geom::Point3d.new 0,0,0
	xvector = Geom::Vector3d.new 1,0,0
	yvector = Geom::Vector3d.new 0,1,0
	zvector = Geom::Vector3d.new 0,0,1
	nangle = si["NorthAngle"].degrees
	tr = Geom::Transformation.rotation point, zvector, -nangle
	tr2 = Geom::Transformation.rotation point, zvector, nangle
	ptx=xvector.transform! tr     
	pty=yvector.transform! tr     
	v=v.transform! tr2		#Sun direction has to be rotated in opposite direction..
	shadow = Sketchup.active_model.shadow_info.to_a

	if ($sun==true) and ($environment=="Sun and Sky")

	
	
		out.puts "		<skylight>"
		out.puts "			<sundir>"+ "%.4f" % (v.x) + " " + "%.4f" % (v.y) + " " + "%.4f" % (v.z) + "</sundir>"
		out.puts "			<turbidity>#{$turbidity}</turbidity>"
		out.puts "		</skylight>"


	else
		if $environment=="Background"

			result=Sketchup.active_model.rendering_options["BackgroundColor"]
			
			if result
				red=result.red*200000
				green=result.green*200000
				blue=result.blue*200000
			else
				red=0.0
				green=0.0
				blue=0.0
			end
			
			out.puts "		<background>"
			out.puts "			<spectrum>"
			out.puts "				<rgb>"
			out.print "					<rgb>#{red} #{green} #{blue}</rgb>\n"
			out.puts "				</rgb>"
			out.puts "			</spectrum>"
			out.puts "		</background>"
		end
		
		if $environment=="Environment Map"
			UI.messagebox "Warning:No Environment Map has been set." if $exr==""
			out.puts "		<env_map>"
			if $exr_type=="Lat_Long"
				out.puts "			<latlong>"
				#out.puts "				<path>probes\\#{$exr}</path>"
				out.puts "				<path>#{$exr}</path>"
				out.puts "				<gain>#{$exr_gain}</gain>"
				out.puts "			</latlong>"
				out.puts "		</env_map>"
			else
				if $exr_type=="Spherical"
				out.puts "			<spherical>"
				#out.puts "				<path>probes\\#{$exr}</path>"
				out.puts "				<path>#{$exr}</path>"
				out.puts "				<width>#{$exr_width}</width>"
				out.puts "				<gain>#{$exr_gain}</gain>"
				out.puts "			</spherical>"
				out.puts "		</env_map>"
				end
			end
		end
	end

end


#### -------------- Export rendering settings ----------------- ##########

def SU2IN::export_global_settings(out)

	

bidir = "true" if $bidir=="On"
bidir = "false" if $bidir=="Off"


out.puts "		<renderer_settings>"
out.puts "			<width>#{$width}</width>"
out.puts "			<height>#{$height}</height>"
out.puts "			<max_num_consec_rejections>#{$mncr}</max_num_consec_rejections>"
#####################################bidirectional currently broken
out.puts "			<bidirectional>false</bidirectional>"
#out.puts "			<bidirectional>#{bidir}</bidirectional>"
out.puts "			<halt_time>#{$halt_time}</halt_time>"
out.puts "			<image_save_period>#{$image_save_period}</image_save_period>"
out.puts "			<num_threads>#{$threads}</num_threads>" if $threads !="Low Priority"
out.puts "		</renderer_settings>"


out.puts "		<tonemapping>"
out.puts "			<reinhard>"
out.puts "				<pre_scale>1.0</pre_scale>"
out.puts "				<post_scale>1.0</post_scale>"
out.puts "				<burn>6.0</burn>"
out.puts "			</reinhard>"
out.puts "		</tonemapping>"



end







##### --- EXPORT MATERIAL -----#####
def SU2IN::export_mat(out,mat)



if mat !=nil
	mat_name=mat.display_name
	mat_name=mat_name.gsub(/[<>]/, '*') 
		
end


mat.attribute_dictionary 'indigo07_materials' if mat
if mat
	
	if mat.alpha<1.0
		type=mat.get_attribute 'indigo07_materials','type',"Specular"
	else
		type=mat.get_attribute 'indigo07_materials','type',"Diffuse"
	end
end



if (mat==nil) or ($default_material_exported==false)
		colour = Sketchup.active_model.rendering_options["FaceFrontColor"]
		result=colour.red,colour.green,colour.blue
		result=[216,216,216] if $g_su2in_clay==true
		out.puts "		<material>"
		out.print "			<name>Default</name>\n"
		out.print "			<diffuse>\n"
		out.puts  "				<colour>" + "%.4f" %((result[0]/255.0)) + " "+ "%.4f" % ((result[1]/255.0)) + " "+ "%.4f" % ((result[2]/255.0)) + "</colour>"
		out.print " 			</diffuse>\n"
		out.print " 		</material>\n"
		$default_material_exported=true
end


	

	
	#export specular material 

	if (mat != nil and type=="Specular")
		bump=mat.get_attribute 'indigo07_materials','bump_map',"None"
		result=mat.color.red,mat.color.green,mat.color.blue
	
		#alpha_factor=mat.alpha*500
		alpha_factor=mat.alpha*mat.alpha*1000

		sss_spectrum=[]
		sss_spectrum=(mat.get_attribute 'indigo07_materials','sssval',"1000").to_s.split(' ')
		if sss_spectrum[0] and sss_spectrum[1] and sss_spectrum[2]
			rgb_spectrum=true
		else
			rgb_spectrum=false
		end
		
		#Export the medium
		medium_name=mat_name+"_medium"
		out.puts "		<medium>"
		out.print "			<name>#{medium_name}</name>\n"
		out.print "			<precedence>#{mat.get_attribute 'indigo07_materials','precedence',10}</precedence>\n"
		out.print "			<basic>\n"
		out.print "				<ior>#{mat.get_attribute 'indigo07_materials','ior',1.52}</ior>\n"
		out.print "				<cauchy_b_coeff>#{mat.get_attribute 'indigo07_materials','cau',0.0}</cauchy_b_coeff>\n"
		out.puts "				<absorption_coefficient_spectrum>"
		out.puts "					<rgb>"
		out.print "						<rgb>#{(1.0-result[0]/255.0)*alpha_factor} #{(1.0-result[1]/255.0)*alpha_factor} #{(1.0-result[2]/255.0)*alpha_factor}</rgb>\n"
		out.print "					</rgb>\n"
		out.print "				</absorption_coefficient_spectrum>\n"
		if (mat.get_attribute 'indigo07_materials','sss',"No")=="Yes"
			out.print "				<subsurface_scattering>\n"
			out.print "					<scattering_coefficient_spectrum>\n"
			
			if rgb_spectrum==true
				out.print "						<rgb>\n"
				out.print "							<rgb>#{sss_spectrum[0]} #{sss_spectrum[1]} #{sss_spectrum[2]}</rgb>\n"
				out.print "						</rgb>\n"
			else
				out.print "						<uniform>\n"
				out.print "							<value>#{sss_spectrum[0]}</value>\n"
				out.print "						</uniform>\n"
			end

			out.print "					</scattering_coefficient_spectrum>\n"
			out.print "					<phase_function>\n"
			out.print "						<uniform/>\n"
			out.print "					</phase_function>\n"
			out.print "				</subsurface_scattering>\n"
			
		end
		out.print "			</basic>\n"
		out.print "		</medium>\n"
		
		#Export the material
	
		out.print "		<material>\n"
		out.print " 		<name>#{mat_name}</name>\n"
		out.print " 		<specular>\n"
		out.print " 			<transparent>true</transparent>\n" if mat.alpha<1.0
		out.print "			<transparent>false</transparent>\n" if mat.alpha==1.0
		out.print "			<internal_medium_name>#{medium_name}</internal_medium_name>\n"

		SU2IN.export_bump_map(out,mat,bump) if bump!="None"
		
		out.print " 		</specular>\n"
		out.print " 		</material>\n"
		return
	end

	#export glossy transparent material

	if (mat != nil and type=="Glossy Transparent")
		bump=mat.get_attribute 'indigo07_materials','bump_map',"None"
		exp_map=mat.get_attribute 'indigo07_materials','exp_map',"None"
		result=mat.color.red,mat.color.green,mat.color.blue
		alpha_factor=mat.alpha*mat.alpha*1000
		
		#Export the medium
		medium_name=mat_name+"_medium"
		out.puts "		<medium>"
		out.print "			<name>#{medium_name}</name>\n"
		out.print "			<precedence>#{mat.get_attribute 'indigo07_materials','precedence',10}</precedence>\n"
		out.print "			<basic>\n"
		out.print "				<ior>#{mat.get_attribute 'indigo07_materials','ior',1.52}</ior>\n"
		out.print "				<cauchy_b_coeff>#{mat.get_attribute 'indigo07_materials','cau',0.0}</cauchy_b_coeff>\n"
		out.puts "				<absorption_coefficient_spectrum>"
		out.puts "					<rgb>"
		out.print "						<rgb>#{(1.0-result[0]/255.0)*alpha_factor} #{(1.0-result[1]/255.0)*alpha_factor} #{(1.0-result[2]/255.0)*alpha_factor}</rgb>\n"
		out.print "					</rgb>\n"
		out.print "				</absorption_coefficient_spectrum>\n"
		if (mat.get_attribute 'indigo07_materials','sss',"No")=="Yes"
			out.print "				<subsurface_scattering>\n"
			out.print "					<scattering_coefficient_spectrum>\n"
			out.print "						<uniform>\n"
			out.print "							<value>#{mat.get_attribute 'indigo07_materials','sssval',"1000"}</value>\n"
			out.print "						</uniform>\n"
			out.print "					</scattering_coefficient_spectrum>\n"
			out.print "					<phase_function>\n"
			out.print "						<uniform/>\n"
			out.print "					</phase_function>\n"
			out.print "				</subsurface_scattering>\n"
			
		end
		out.print "			</basic>\n"
		out.print "		</medium>\n"
		
		#Export the material
	
		out.print "		<material>\n"
		out.print " 		<name>#{mat_name}</name>\n"
		out.print " 		<glossy_transparent>\n"
		out.print "			<internal_medium_name>#{medium_name}</internal_medium_name>\n"
		out.print "			<exponent>#{mat.get_attribute 'indigo07_materials','exp',1000}</exponent>\n"

		SU2IN.export_bump_map(out,mat,bump) if bump!="None"
		SU2IN.export_exp_map(out,mat,exp_map) if exp_map!="None"
		
		out.print " 		</glossy_transparent>\n"
		out.print " 		</material>\n"
		return
	end

#export diffuse material
	
	if (mat !=nil and type=="Diffuse" )
		result=mat.color.red,mat.color.green,mat.color.blue
		bump=mat.get_attribute 'indigo07_materials','bump_map',"None"
		disable_texture=mat.get_attribute 'indigo07_materials','dis_tex',"No"
		
		out.puts "		<material>"
		out.print "			<name>#{mat_name}</name>\n"
		out.print "			<diffuse>\n"
		if (mat.texture == nil) or (disable_texture=="Yes")
			out.puts  "				<colour>" + "%.4f" %((result[0]/255.0)) + " "+ "%.4f" % ((result[1]/255.0)) + " "+ "%.4f" % ((result[2]/255.0)) + "</colour>"
		end
			
		if (mat.texture) and (disable_texture)!="Yes"
			out.print "				<colour>1.0 1.0 1.0</colour>\n"
			out.print "				<albedo_texture>\n"
			out.print "					<uv_set>uv</uv_set>\n"
			out.print "					<path>#{SU2IN.trunc_path(mat.texture.filename)}</path>\n"
			out.print " 					<exponent>2.2</exponent>\n"
			out.print " 				</albedo_texture>\n"
		end
		SU2IN.export_bump_map(out,mat,bump) if bump!="None"

		
		out.print "			</diffuse>\n"
		out.print "		</material>\n"
		return
	end

	#export phong material

	if (mat !=nil and type== "Phong" )
		result=mat.color.red,mat.color.green,mat.color.blue
		bump=mat.get_attribute 'indigo07_materials','bump_map',"None"
		exp_map=mat.get_attribute 'indigo07_materials','exp_map',"None"
		disable_texture=mat.get_attribute 'indigo07_materials','dis_tex',"No"

		
		out.puts "		<material>"
		out.print "			<name>#{mat_name}</name>\n"
		out.print "			<phong>\n"
		if (mat.texture == nil) or (disable_texture=="Yes")
			out.puts  "				<diffuse>" + "%.4f" %((result[0]/255.0)) + " "+ "%.4f" % ((result[1]/255.0)) + " "+ "%.4f" % ((result[2]/255.0)) + "</diffuse>"
			out.puts "				<ior>#{mat.get_attribute 'indigo07_materials','ior',1.52}</ior>\n"
			out.print " 				<fresnel_scale>#{mat.get_attribute 'indigo07_materials','fres',1.0}</fresnel_scale>\n"
			out.print " 				<exponent>#{mat.get_attribute 'indigo07_materials', 'exp' ,1000}</exponent>\n"
		end
		
		if (mat.texture) and (disable_texture)!="Yes"
			out.puts  "				<diffuse>1.0 1.0 1.0</diffuse>"
			out.puts "				<ior>#{mat.get_attribute 'indigo07_materials','ior',1.52}</ior>\n"
			out.print " 				<fresnel_scale>#{mat.get_attribute 'indigo07_materials','fres',1.0}</fresnel_scale>\n"
			out.print " 				<exponent>#{mat.get_attribute 'indigo07_materials', 'exp' ,1000}</exponent>\n"
			out.print " 				<albedo_texture>\n"
			out.print "					<uv_set>uv</uv_set>\n"
			out.print "					<path>#{SU2IN.trunc_path(mat.texture.filename)}</path>\n"
			out.print " 					<exponent>2.2</exponent>\n"
			out.print " 				</albedo_texture>\n"
		end

		SU2IN.export_bump_map(out,mat,bump) if bump!="None"
		SU2IN.export_exp_map(out,mat,exp_map) if exp_map!="None"

		out.print " 			</phong>\n"
		out.print " 		</material>\n"	
		return
	end
	
	#export NK_data materials
	
	if (mat !=nil)
		if (type=="Al") or (type=="Cr") or (type=="Cu") or (type=="Au") or (type=="Ag") or (type=="Ti") or (type=="W") or (type=="Li") or (type=="Ni") or (type=="Pt") 
			exponent=mat.get_attribute 'indigo07_materials','exp',"1000"
			bump=mat.get_attribute 'indigo07_materials','bump_map',"None"
			exp_map=mat.get_attribute 'indigo07_materials','exp_map',"None"
		
			out.puts "		<material>"
			out.puts "		<name>#{mat_name}</name>"
			out.puts "			<phong>"
			out.puts "				<nk_data>nkdata/#{type}.nk</nk_data>"
			out.puts "				<exponent>#{exponent}</exponent>"
			
			SU2IN.export_bump_map(out,mat,bump) if bump!="None"
			SU2IN.export_exp_map(out,mat,exp_map) if exp_map!="None"		
		
			out.puts "			</phong>"
			out.puts "		</material>"
		end
	end
	
	#export blend material

	if (mat !=nil and type=="Blend")
		disable_texture=mat.get_attribute 'indigo07_materials','dis_tex',"No"
		matA=mat.get_attribute 'indigo_blend','matA'
		matB=mat.get_attribute 'indigo_blend','matB'
		materials = Sketchup.active_model.materials
   		translatednames = materials.collect {|m| m.display_name}
		indexA = translatednames.index(matA)
		indexB = translatednames.index(matB)
		if (indexA==nil and matA!="None")
			UI.messagebox ("Error: Blend material tried to use non-existent material named") + " #{matA}"
		end
		if (indexB==nil and matB!="None")
			UI.messagebox ("Error: Blend material tried to use non-existent material named") + " #{matB}"
		end
		matA=matA.gsub(/[<>]/, '*')
		matB=matB.gsub(/[<>]/, '*')
		if ((matA=="None") or (matB=="None")) and $null_exported==false
			out.puts "		<material>"
			out.puts "			<name>None</name>"
			out.puts "			<null_material/>"
			out.puts "		</material>"
			$null_exported=true
		end
				
		out.puts "		<material>"
		out.print "			<name>#{mat_name}</name>\n"
		out.print "			<blend>\n"
		out.print "				<a_name>#{matA}</a_name>\n"
		out.print "				<b_name>#{matB}</b_name>\n"
		out.print "				<blend_factor>#{mat.get_attribute 'indigo_blend','blendfactor'}</blend_factor>\n"
		
		if (mat.texture) and (disable_texture)!="Yes" 
			out.print "				<blend_map>\n"
			out.print "					<uv_set>uv</uv_set>\n"
			out.print "					<path>#{SU2IN.trunc_path(mat.texture.filename)}</path>\n"
			out.print " 					<exponent>1.0</exponent>\n"
			out.print " 				</blend_map>\n"
		end
		
		out.print "			</blend>\n"
		out.print "		</material>\n"
		return
	end
	
end

##### --- EXPORT BUMP MAP -----######

def SU2IN::export_bump_map(out,mat,bm)
 	
	model = Sketchup.active_model
 	materials = model.materials
  	translatednames = materials.collect {|m| m.display_name}
	names=materials.collect {|m| m.name}
	bump_index=translatednames.index(bm)
	bump_material=bump_index ? materials[names[bump_index]] : nil
		if (not bump_material)
			UI.messagebox ("Could not find Bump Map Material named") + " #{bm}"
			return
		end
	#bump_texture=bump_material.texture.filename if bump_material.texture
	
	

	out.print "				<bump_map>\n"
	out.print "					<uv_set>uv</uv_set>\n"
	out.print "					<path>#{SU2IN.trunc_path(bump_material.texture.filename)}</path>\n"
	out.print "					<b>#{mat.get_attribute 'indigo07_materials','bump_value'}</b>\n"
	out.print "					<exponent>1.0</exponent>\n"
	out.print " 				</bump_map>\n"
end 

##### --- EXPORT EXPONENT MAP -----######

def SU2IN::export_exp_map(out,mat,em)
 	
	model = Sketchup.active_model
 	materials = model.materials
  	translatednames = materials.collect {|m| m.display_name}
	names=materials.collect {|m| m.name}
	exp_index=translatednames.index(em)
	exp_material=exp_index ? materials[names[exp_index]] : nil
		if (not exp_material)
			UI.messagebox ("Could not find Exponent Map Material named") + " #{em}"
			return
		end
	
	out.print "				<exponent_map>\n"
	out.print "					<uv_set>uv</uv_set>\n"
	out.print "					<path>#{SU2IN.trunc_path(exp_material.texture.filename)}</path>\n"
	out.print "					<b>#{mat.get_attribute 'indigo07_materials','expmap_value'}</b>\n"
	out.print "					<exponent>1.0</exponent>\n"
	out.print " 				</exponent_map>\n"
end 

##### --- EXPORT CLAY MATERIAL -----#####

def SU2IN::export_clay(out)
	
	out.puts "		<material>"
	out.print "			<name>Clay</name>"
	out.print "			<diffuse>"
	out.print "				<colour>0.85 0.85 0.76</colour>" 
	out.print "			</diffuse>"
	out.print " 	</material>"

end


#### - Send text to status bar - ####
def SU2IN::status_bar(stat_text)
	
	statbar = Sketchup.set_status_text stat_text
	
end




def SU2IN::trunc_path(s)
	j=0
	for i in (0..s.length)
		if s[i,1] == "\\" or s[i,1] == "/"
			j=i
		end 
	end
	if j != 0
	s=s[j+1..s.length]
	end
		if s[s.length-3,3]=="jpg" or s[s.length-3,3]=="JPG" or s[s.length-3,3]=="png" or s[s.length-3,3]=="tif"
		else
		s[s.length-3..s.length]="png"
		end

	s=$g_su2in_textures_prefix+$g_su2in_model_name+"\\"+s
	s
end

#### -------- Mesh export ---------- ######

def SU2IN::export_mesh(export_name)
	
	$parent_mat=[]
	
	if $g_su2in_export_meshes == true
		
		
		SU2IN.export_entities(Sketchup.active_model.entities, Geom::Transformation.new)
		SU2IN.export_faces(export_name)
		SU2IN.export_fm_faces(export_name)

	end	
end

###########

def SU2IN::export_faces(export_name)

	for mat in Sketchup.active_model.materials
		if ($g_su2in_materials[mat]!=nil and $g_su2in_materials[mat]!=[])
			SU2IN.export_face(export_name,mat,false)
			$g_su2in_materials[mat]=[]
		end
	end
	if ($g_su2in_materials[nil]!=nil and $g_su2in_materials[nil]!=[])
		SU2IN.export_face(export_name,nil,false)
		$g_su2in_materials[nil]=[]
	end

end

###########

def SU2IN::export_fm_faces(export_name)


		for mat in Sketchup.active_model.materials
			if ($g_su2in_fm_materials[mat]!=nil and $g_su2in_fm_materials[mat]!=[])
				SU2IN.export_face(export_name,mat,true)
				$g_su2in_fm_materials[mat]=[]
			end
		end
		if ($g_su2in_fm_materials[nil]!=nil and $g_su2in_fm_materials[nil]!=[])
			SU2IN.export_face(export_name,nil,true)
			$g_su2in_fm_materials[nil]=[]
		end

end

##### ------------ Exporting entities to an array -------------- ##########

def SU2IN::export_entities(object, trans)

if object.class == Sketchup::ComponentInstance

	entity_list=object.definition.entities 

elsif object.class == Sketchup::Group

	entity_list=object.entities 
else
	entity_list=object
end
text=""
text="Component: " + object.definition.name if object.class == Sketchup::ComponentInstance
text="Group" if object.class == Sketchup::Group

SU2IN.status_bar("Collecting Faces - Level #{$parent_mat.size} - #{text}")  #{$face}

	for e in entity_list
		if (e.class == Sketchup::Group and e.visible? and e.layer.visible?)
			$fm_comp.push(false)

			if e.material != nil
				mat = e.material
				$parent_mat.push(e.material)
				SU2IN.store_textured_entities(e)			
			else
				$parent_mat.push($parent_mat.last)
			end

			SU2IN.export_entities(e, trans*e.transformation)
			
			$parent_mat.pop
			$fm_comp.pop
		end
		if (e.class == Sketchup::ComponentInstance and e.visible? and e.layer.visible?)

			$fm_comp.push(e.definition.behavior.always_face_camera?)

				if e.material != nil
					mat = e.material
					$parent_mat.push(e.material)
					SU2IN.store_textured_entities(e)
				else
					$parent_mat.push($parent_mat.last)
				end
			
			SU2IN.export_entities(e, trans*e.transformation)
			$parent_mat.pop
			$fm_comp.pop
				
		end
		if (e.class == Sketchup::Face and e.visible? and e.layer.visible?)

			if e.material == nil
				mat=$parent_mat.last				
			else
				SU2IN.store_textured_entities(e)
				mat = e.material
			end

			if $g_su2in_materials[mat] == nil
				$g_su2in_materials[mat] = []
			end

			if $g_su2in_fm_materials[mat] == nil
				$g_su2in_fm_materials[mat] = []
			end


			if $fm_comp.last==true
				$g_su2in_fm_materials[mat] = $g_su2in_fm_materials[mat] + [[e,trans]]
			else
				$g_su2in_materials[mat] = $g_su2in_materials[mat] + [[e,trans]] if $animation==false
			end

			#$face+=1
		end
	end

return nil
end

def SU2IN::store_textured_entities(e)
	if (e.material.respond_to?(:texture) and e.material.texture !=nil)
		saved=false
		for i in $g_su2in_model_textures
			saved=true if i[0]== e.material
		end
		if saved==false
			$g_su2in_model_textures.push([e.material, e])
		end
	end
end

def SU2IN::point_to_vector(p)
	Geom::Vector3d.new(p.x,p.y,p.z)
end

##### ---- Exporting polymeshes ---- #######

def SU2IN::export_face(out,mat,fm_mat)

	

	meshes = []
	polycount = 0
	pointcount = 0
	mirrored={}
	if fm_mat
		export=$g_su2in_fm_materials[mat]
	else
		export=$g_su2in_materials[mat]
	end

	for ft in export
		polymesh = ft[0].mesh 5 #
		polymesh.transform! ft[1]
		mirrored[polymesh] = true
		xa = SU2IN.point_to_vector(ft[1].xaxis)
		ya = SU2IN.point_to_vector(ft[1].yaxis)
		za = SU2IN.point_to_vector(ft[1].zaxis)
		xy = xa.cross(ya)
		xz = xa.cross(za)
		yz = ya.cross(za)
		if xy.dot(za) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		if xz.dot(ya) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		if yz.dot(xa) < 0
			mirrored[polymesh] = !mirrored[polymesh]
		end
		meshes = meshes + [polymesh]

		polycount=polycount + polymesh.count_polygons
		pointcount=pointcount + polymesh.count_points
	end

	startindex = 0
	has_texture = false
	if mat
		matname = mat.display_name
		matname=matname.gsub(/[<>]/, '*') 
		
		if mat.texture
			has_texture = true
		end
	else
		matname = "Default"
	end

#matname="FM_"+matname if fm_mat

#Exporting verticies

if mat
	SU2IN.status_bar("Material being exported: " + mat.name + " - Verticies")
end

out.puts "		<mesh>"
out.puts "			<name>#{matname}</name>"
if mat
	out.print "			<normal_smoothing>false</normal_smoothing>\n" if ((mat.get_attribute 'indigo07_materials','no_smooth',"No")=="Yes")
	out.print "			<normal_smoothing>true</normal_smoothing>\n" if ((mat.get_attribute 'indigo07_materials','no_smooth',"No")=="No")
else
	out.print "			<normal_smoothing>true</normal_smoothing>\n" 
end

out.puts " 			<embedded>"
out.puts "				<expose_uv_set>"
out.puts "					<index>0</index>"
out.puts "					<name>uv</name>"
out.puts "				</expose_uv_set>"

	for mesh in meshes
		for p in (1..mesh.count_points)
			pos = mesh.point_at(p).to_a
			norm = mesh.normal_at(p).to_a
			if (has_texture and $g_su2in_clay==false)
				if has_texture and mat==nil
					texsize = Geom::Point3d.new(mat.texture.width, mat.texture.height, 1)
				else
					texsize = Geom::Point3d.new(1,1,1)
				end
				#uvq = Sketchup::UVHelper.get_front_UVQ mesh.point_at(p)
				#uv = [uvq.x/texsize.x, uvq.y/texsize.y, uvq.z/texsize.z]
				
				uv = [mesh.uv_at(p,1).x/texsize.x, mesh.uv_at(p,1).y/texsize.y, mesh.uv_at(p,1).z/texsize.z]
				if has_texture 
					out.print "				<vertex pos=\"#{"%.4f" %(pos[0]*$g_su2in_scale)} #{"%.4f" %(pos[1]*$g_su2in_scale)} #{"%.4f" %(pos[2]*$g_su2in_scale)}\" normal=\"#{"%.4f" %(norm[0])} #{"%.4f" %(norm[1])} #{"%.4f" %(norm[2])}\" uv0=\"#{"%.4f" %(uv[0])} #{"%.4f" %(uv[1]+1)}\"/>\n"
				end
			else
				out.print "				<vertex pos=\"#{"%.4f" %(pos[0]*$g_su2in_scale)} #{"%.4f" %(pos[1]*$g_su2in_scale)} #{"%.4f" %(pos[2]*$g_su2in_scale)}\" normal=\"#{"%.4f" %(norm[0])} #{"%.4f" %(norm[1])} #{"%.4f" %(norm[2])}\"/>\n"
			end
		end
	end

	#Exporting faces

	SU2IN.status_bar("Material being exported: " + mat.name + " - Faces") if mat

			out.print "				<triangle_set>\n"
			if ($g_su2in_clay==true) or ($monochrome==true)
				out.print "					<material_name>#{$monochrome_material}</material_name>\n"
			else
				out.print "					<material_name>#{matname}</material_name>\n"
			end
	for mesh in meshes
		for poly in mesh.polygons
			v1 = (poly[0]>=0?poly[0]:-poly[0])+startindex
			v2 = (poly[1]>=0?poly[1]:-poly[1])+startindex
			v3 = (poly[2]>=0?poly[2]:-poly[2])+startindex
			if !mirrored[mesh]
				out.print "					<tri>#{v1-1} #{v2-1} #{v3-1}</tri>\n"
			else
				out.print "					<tri>#{v2-1} #{v1-1} #{v3-1}</tri>\n"
			end
			$g_su2in_count_tri = $g_su2in_count_tri + 1
		end
		startindex = startindex + mesh.count_points
	end

			out.puts "				</triangle_set>"
			out.puts "			</embedded>"
			out.puts "		</mesh>"

#Exporting Material
	
	
		
		
		
       $g_su2in_count_material = $g_su2in_count_material + 1
	type=mat.get_attribute 'indigo07_materials','type',"Diffuse" if mat
	
	if (mat != nil and type=="Blackbody Light")
		$n_meshlights+=1
		
		temp=mat.get_attribute 'indigo07_materials','temp',0.0
		power=mat.get_attribute 'indigo07_materials','power'
		ole=mat.get_attribute 'indigo07_materials','ole',17.5
		
		out.print "		<meshlight>\n"
		out.print "			<pos>0.000000 0.000000 0.000000</pos>\n"	
		out.print "			<scale>1.0</scale>\n"	
		out.print "			<rotation>\n"
		out.print "				<matrix>\n"
		out.print "					1.0 0.000000 0.000000 0.000000 1.0 0.000000 0.000000 0.000000 1.0\n"
		out.print "				</matrix>\n"
		out.print "			</rotation>\n"
		out.print " 			<mesh_name>#{matname}</mesh_name>\n"
		out.print " 			<spectrum>\n"
		out.print "				<blackbody>\n"
		out.print " 					<temperature>#{temp}</temperature>\n"
		out.print "					<gain>1.0</gain>\n"
		out.print "				</blackbody>\n"
		out.print "			</spectrum>\n"
		out.print "			<efficacy_scale>\n"
		out.print "				<power_drawn>#{power}</power_drawn>\n"
		out.print "				<overall_luminous_efficacy>#{ole}</overall_luminous_efficacy>\n"
		out.print "			</efficacy_scale>\n"
		out.print "		</meshlight>\n"
		
	end


	#export RGB Meshlight
	
	if (mat != nil and type=="RGB Light")
		$n_meshlights+=1
		
		result=mat.color.red,mat.color.green,mat.color.blue
		power=mat.get_attribute 'indigo07_materials','power'
		ole=mat.get_attribute 'indigo07_materials','ole',17.5
		out.print "		<meshlight>\n"
		out.print "			<pos>0.000000 0.000000 0.000000</pos>\n"	
		out.print "			<scale>1.0</scale>\n"	
		out.print "			<rotation>\n"
		out.print "				<matrix>\n"
		out.print "					1.0 0.000000 0.000000 0.000000 1.0 0.000000 0.000000 0.000000 1.0\n"
		out.print "				</matrix>\n"
		out.print "			</rotation>\n"
		out.print " 			<mesh_name>#{matname}</mesh_name>\n"
		out.print " 			<spectrum>\n"
		out.print "				<rgb>\n"
		out.print " 					<rgb>#{result[0]/255.0**2.2} #{result[1]/255.0**2.2} #{result[2]/255.0**2.2}</rgb>\n"
		out.print "				</rgb>\n"
		out.print "			</spectrum>\n"
		out.print "			<efficacy_scale>\n"
		out.print "				<power_drawn>#{power}</power_drawn>\n"
		out.print "				<overall_luminous_efficacy>#{ole}</overall_luminous_efficacy>\n"
		out.print "			</efficacy_scale>\n"
		out.print "		</meshlight>\n"
		
	end
	if (mat and (type != "RGB Light") and (type != "Blackbody Light"))
		out.print "		<model>\n"
		out.print "			<pos>0.0 0.0 0.0</pos>\n"
		out.print "			<scale>1.0</scale>\n"	
		out.print "			<rotation>\n"
		out.print "				<matrix>\n"
		out.print "					1.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0 1.0 \n"
		out.print "				</matrix>\n"
		out.print "			</rotation>\n"
		out.print "			<mesh_name>#{matname}</mesh_name>\n"
		out.print "		</model>\n"
	end
	if (mat==nil)
		out.print "		<model>\n"
		out.print "			<pos>0.0 0.0 0.0</pos>\n"
		out.print "			<scale>1.0</scale>\n"	
		out.print "			<rotation>\n"
		out.print "				<matrix>\n"
		out.print "					1.0 0.0 0.0 0.0 1.0 0.0 0.0 0.0 1.0 \n"
		out.print "				</matrix>\n"
		out.print "			</rotation>\n"
		out.print "			<mesh_name>#{matname}</mesh_name>\n"
		out.print "		</model>\n"
	end

mirrored={}
meshes = []
end

### ------ Function RGB_2_HSB -------- ###
def SU2IN::rgb_to_hsb(r, g, b)

r/=255.0
g/=255.0
b/=255.0

  rgb_max = [r,g,b].max
  rgb_min = [r,g,b].min

  v = rgb_max.to_f

  if rgb_max != 0
    	s = (rgb_max - rgb_min).to_f / (rgb_max).to_f
  else
    	s = 0.0
	h = 0.0
	return h,s,v*100.0
  end

if r==g && g==b
	h=0     
else
    h = ( g-b )/ (rgb_max - rgb_min).to_f if r == rgb_max
    
    h = 2.0 + (b-r).to_f/(rgb_max - rgb_min).to_f if g == rgb_max

    h = 4.0 + (r-g).to_f/(rgb_max - rgb_min).to_f if b == rgb_max
end

h+=6 if h<0
return ((h*600.0).round)/10.0,((s*1000.0).round)/10.0,((v*1000.0).round)/10.0
end


### ------ Function HSB_2_RGB -------- ###

def SU2IN::hsb_to_rgb(h,s,v)
 
	if ((s == 0) && (h == 0))
		r = g = b = v
	end
	v/=100.0
	s/=100.0
	h = 0.0 if (h == 360)
	
	h /= 60.0
	i = h.floor
	f = h - i
	p = v*(1 - s)
	q = v*(1 - s*f)
	t = v*(1 - s*(1 - f))

	case i
	when 0
		r = v
 		g = t
		b = p

	when 1
		r = q
		g = v
		b = p
	when 2
		r = p
		g = v
		b = t
	when 3
		r = p
		g = q
		b = v
	when 4
		r = t
		g = p
		b = v
	when 5
		r = v
		g = p
		b = q
	end

r*=255.0		
g*=255.0
b*=255.0

return r.round,g.round,b.round

end



#Assigning Indigo Material Properties------------------------------------------------
def SU2IN::assign_properties_to_material

	SU2IN.reset_global_variables
	textures=""
	texture_list=["None"]

# First get a list of all of the materials in the model
    
	model = Sketchup.active_model
	ent=model.selection.first
	$selected_material=ent.material if ent !=nil
	

    materials = model.materials
    names = materials.collect {|m| m.name}
    translatednames = materials.collect {|m| m.display_name}
    presets=["Keep Current","DIFFUSE","PHONG","SPECULAR","GLOSSY TRANSPARENT","BLEND","METALS","Glass","Frosted Glass","Mirror","Ceramic","Dull Plastic","Shiny Plastic","Leather","Car Paint","Curtains","Lampshade","RGB LIGHT","BLACKBODY LIGHT","100W Incandescent","60W Incandescent","40W Incandescent","5W Incandescent"]

#Get Array of Materials with Textures

    materials.each do |m| 
        if (m.respond_to?(:texture) and m.texture !=nil)
		textures+=(m.display_name)+"|"
	end
    next
	
    end 


texture_list=textures.split('|') if textures !=""
texture_list.unshift("None") if texture_list != ["None"]
	

    
# Display a dialog to pick a material and preset
    prompts = ["Material Name", "Material Preset"]
    if $selected_material != nil
	values = [$selected_material.display_name,presets[0]]
	else
	values = [translatednames[0], presets[0]]
    end
    enums = [translatednames.join("|"),presets.join("|")]
    results = inputbox prompts, values, enums,("Select Material")
    return if not results

	
	current_mat=results[0]
	current_preset=results[1]

# Get the selected Material
    index = translatednames.index(current_mat)
		
    material = index ? materials[names[index]] : nil
    if( not material )
        UI.messagebox ("Could not find Material named") + " #{results[0]}"
        return
    end

current_bump=material.get_attribute 'indigo07_materials','bump_map',"None"
current_exp=material.get_attribute 'indigo07_materials','exp_map',"None"

# Display a dialog to pick material properties

if current_preset=="Keep Current"
	
	if material.alpha<1.0
		current_preset=material.get_attribute 'indigo07_materials','type',"Glass"
		current_preset=current_preset.upcase if current_preset!="Glass"
	else
		current_preset=material.get_attribute 'indigo07_materials','type',"Diffuse"
		if (current_preset=="Al") or (current_preset=="Cr") or (current_preset=="Cu") or (current_preset=="Au") or (current_preset=="Ag") or (current_preset=="Ti") or (current_preset=="W") or (current_preset=="Li") or (current_preset=="Ni") or (current_preset=="Pt")
			current_preset="METALS"
		else
			current_preset=current_preset.upcase
		end
	end
end


case current_preset

    	when "Glass"
		current_preset=SU2IN.apply_glass(material)
	when "Frosted Glass"
		current_preset=SU2IN.apply_frosted_glass(material)
	when "Mirror"
		current_preset=SU2IN.apply_mirror(material)
	when "Ceramic"
		current_preset=SU2IN.apply_ceramic(material)
	when "Dull Plastic"
		current_preset=SU2IN.apply_dull_plastic(material)
	when "Shiny Plastic"
		current_preset=SU2IN.apply_shiny_plastic(material)
	when "Leather"
		current_preset=SU2IN.apply_leather(material)
	when "Car Paint"
		current_preset=SU2IN.apply_car_paint(material)
	when "Curtains"
		current_preset=SU2IN.apply_curtains(material)
	when "Lampshade"
		current_preset=SU2IN.apply_lampshade(material)
	when "100W Incandescent"
		current_preset=SU2IN.apply_100W(material)
	when "60W Incandescent"
		current_preset=SU2IN.apply_60W(material)
	when "40W Incandescent"
		current_preset=SU2IN.apply_40W(material)
	when "5W Incandescent"
		current_preset=SU2IN.apply_5W(material)

end

if current_preset=="DIFFUSE"
	
	disable_smoothing=%w[Yes No].join("|")
	disable_mat_texture = %w[Yes No].join("|")
	prompts = ["Bump Map", "Bump Gain","Disable Material Texture","Disable Smoothing"]
	values = [current_bump , (material.get_attribute 'indigo07_materials','bump_value',0.0),(material.get_attribute 'indigo07_materials','dis_tex',"No"),(material.get_attribute 'indigo07_materials','no_smooth',"No")]
	enums=[texture_list.join("|"),nil,disable_mat_texture,disable_smoothing]
	results=inputbox prompts,values, enums,(current_mat+" - Diffuse")
	return if not results
	bump_map = results[0]
	bump_value=results[1]
	disable_mat_texture=results[2]
	disable_smoothing=results[3]
	type="Diffuse"
	
		
end

if current_preset=="PHONG"
	disable_smoothing=%w[Yes No].join("|")
	disable_mat_texture = %w[Yes No].join("|")
	prompts = ["Index of Refraction (>=1.0)", "Exponent (>=0)", "Fresnel Scale (0-1.0)","Bump Map","Bump Gain","Exponent Map","Exponent Map Gain","Disable Material Texture","Disable Smoothing"]
	values = [(material.get_attribute 'indigo07_materials','ior',1.52),(material.get_attribute 'indigo07_materials','exp',1000),(material.get_attribute 'indigo07_materials','fres',1.0),current_bump,(material.get_attribute 'indigo07_materials','bump_value',0.0),current_exp,(material.get_attribute 'indigo07_materials','expmap_value',0.0),(material.get_attribute 'indigo07_materials','dis_tex',"No"),(material.get_attribute 'indigo07_materials','no_smooth',"No")]
	enums=[nil,nil,nil,texture_list.join("|"),nil,texture_list.join("|"),nil,disable_mat_texture,disable_smoothing]
	results=inputbox prompts,values, enums,(current_mat+" - Phong")
	return if not results
	ior = results[0]
	exponent = results[1]
	fresnel = results[2]
	bump_map = results[3]
	bump_value=results[4]
	exp_map=results[5]
	expmap_value=results[6]
	disable_mat_texture=results[7]
	disable_smoothing=results[8]
	type="Phong"
	
	

end

if current_preset=="SPECULAR"
	disable_smoothing=%w[Yes No].join("|")
	sss=%w[Yes No].join("|")
	prompts = ["Index of Refraction (>=1.0)", "Cauchy Dispersion Coefficient (>=0.0)","Precedence (>=1)","Sub-Surface Scattering","Scattering Spectrum","Bump Map","Bump Gain","Disable Smoothing"]
	values = [(material.get_attribute 'indigo07_materials','ior',1.52),(material.get_attribute 'indigo07_materials','cau',0.0),(material.get_attribute 'indigo07_materials','precedence',10),(material.get_attribute 'indigo07_materials','sss',"No"),(material.get_attribute 'indigo07_materials','sssval',1000).to_s,current_bump,(material.get_attribute 'indigo07_materials','bump_value',0.0),(material.get_attribute 'indigo07_materials','no_smooth',"No")]
	enums = [nil,nil,nil,sss,nil,texture_list.join("|"),nil,disable_smoothing]
	results=inputbox prompts,values,enums, (current_mat+" - Specular")
	return if not results
	ior = results[0]
	cauchy = results[1]
	precedence=results[2]
	sss=results[3]
	sss_spectrum=results[4]
	bump_map=results[5]
	bump_value=results[6]
	disable_smoothing=results[7]
	type="Specular"

end

if current_preset=="GLOSSY TRANSPARENT"
	disable_smoothing=%w[Yes No].join("|")
	sss=%w[Yes No].join("|")
	prompts = ["Index of Refraction (>=1.0)","Exponent (>=0)","Cauchy Dispersion Coefficient (>=0.0)","Precedence (>=1)","Sub-Surface Scattering","Scattering Spectrum","Bump Map","Bump Gain","Disable Smoothing"]
	values = [(material.get_attribute 'indigo07_materials','ior',1.52),(material.get_attribute 'indigo07_materials','exp',1000),(material.get_attribute 'indigo07_materials','cau',0.0),(material.get_attribute 'indigo07_materials','precedence',10),(material.get_attribute 'indigo07_materials','sss',"No"),(material.get_attribute 'indigo07_materials','sssval',1000).to_s,current_bump,(material.get_attribute 'indigo07_materials','bump_value',0.0),(material.get_attribute 'indigo07_materials','no_smooth',"No")]
	enums = [nil,nil,nil,nil,sss,nil,texture_list.join("|"),nil,disable_smoothing]
	results=inputbox prompts,values,enums, (current_mat+" - Glossy Transparent")
	return if not results
	ior = results[0]
	exponent=results[1]
	cauchy = results[2]
	precedence=results[3]
	sss=results[4]
	sss_spectrum=results[5]
	bump_map=results[6]
	bump_value=results[7]
	disable_smoothing=results[8]
	type="Glossy Transparent"

end

if current_preset=="BLEND"
	disable_mat_texture = %w[Yes No].join("|")
	disable_smoothing=%w[Yes No].join("|")
	blend_list=["None"]
	for mat in materials
		if ((mat.get_attribute 'indigo07_materials','type',"Diffuse") !="Blackbody Light") and ((mat.get_attribute 'indigo07_materials','type',"Diffuse")!="RGB Light")
			blend_list.push(mat.display_name) if mat!=material
		end
	end
		
	prompts = ["Material A", "Material B","Blend Factor (0-1)","Disable Material Texture","Disable Smoothing"]
	values = [(material.get_attribute 'indigo_blend','matA',"None"), (material.get_attribute 'indigo_blend','matB',"None"),(material.get_attribute 'indigo_blend','blendfactor',0.5),(material.get_attribute 'indigo07_materials','dis_tex',"No"),(material.get_attribute 'indigo07_materials','no_smooth',"No")]
	enums=[blend_list.join("|"),blend_list.join("|"),nil,disable_mat_texture,disable_smoothing]
	results=inputbox prompts,values, enums,(current_mat+" - Blend")
	return if not results

	material.set_attribute 'indigo_blend','matA',results[0]
	material.set_attribute 'indigo_blend','matB',results[1]
	material.set_attribute 'indigo_blend','blendfactor',results[2]
	disable_mat_texture=results[3]
	disable_smoothing=results[4]
	type="Blend"

	
end


if current_preset=="RGB LIGHT"
	prompts = ["Power (Watts)","Overall Luminous Efficacy (lm/W)"]
	values = [(material.get_attribute 'indigo07_materials','power',100.0),(material.get_attribute 'indigo07_materials','ole',17.5)]
	results=inputbox prompts,values,(current_mat+" - RGB Light")
	return if not results
	power=results[0]
	ole=results[1]
	type="RGB Light"
	
		
end

if current_preset=="BLACKBODY LIGHT"
	prompts = ["Power (Watts)","Overall Luminous Efficacy (lm/W)","Temperature (Kelvin)"]
	values = [(material.get_attribute 'indigo07_materials','power',100.0),(material.get_attribute 'indigo07_materials','ole',17.5),(material.get_attribute 'indigo07_materials','temp',5000)]
	results=inputbox prompts,values,(current_mat+" - Blackbody Light")
	return if not results
	power=results[0]
	ole=results[1]
	temp=results[2]
	type="Blackbody Light"


end

if (current_preset=="METALS")
	disable_smoothing=%w[Yes No].join("|")
	nk_data=["Aluminum","Chrome","Copper","Gold","Lithium","Nickel","Platinum","Silver","Titanium","Tungsten"]
	nk_list=nk_data.join("|")
	nk_name=["Al","Cr","Cu","Au","Li","Ni","Pt","Ag","Ti","W"]	
	current_type=material.get_attribute 'indigo07_materials','type',"Al"
	index = nk_name.index(current_type)
	index=0 if not index
	
	prompts = ["Material","Exponent (>=0)","Bump Map","Bump Gain","Exponent Map","Exponent Map Gain","Disable Smoothing"]
	values = [nk_data[index],(material.get_attribute 'indigo07_materials','exp',1000),current_bump,(material.get_attribute 'indigo07_materials','bump_value',0.0),current_exp,(material.get_attribute 'indigo07_materials','expmap_value',0.0),(material.get_attribute 'indigo07_materials','no_smooth',"No")]
	enums=[nk_list,nil,texture_list.join("|"),nil,texture_list.join("|"),nil,disable_smoothing]
	results=inputbox prompts,values,enums,(current_mat+" - Metals")
	return if not results
	index = nk_data.index(results[0])
	type=nk_name[index]
	exponent = results[1]
	bump_map=results[2]
	bump_value=results[3]
	exp_map=results[4]
	expmap_value=results[5]
	disable_smoothing=results[6]
	
	
end

#TODO validate sss spectrum



# And attach the attributes

	material.set_attribute 'indigo07_materials','type', type
   	material.set_attribute 'indigo07_materials', 'exp', exponent
	material.set_attribute 'indigo07_materials', 'fres', fresnel
	material.set_attribute 'indigo07_materials', 'ior', ior
	material.set_attribute 'indigo07_materials', 'cau', cauchy
	material.set_attribute 'indigo07_materials', 'precedence', precedence
	material.set_attribute 'indigo07_materials', 'sss', sss
	material.set_attribute 'indigo07_materials', 'sssval', sss_spectrum
	material.set_attribute 'indigo07_materials', 'power', power
	material.set_attribute 'indigo07_materials', 'ole' , ole
	material.set_attribute 'indigo07_materials', 'bump_map',bump_map
	material.set_attribute 'indigo07_materials', 'bump_value',bump_value
	material.set_attribute 'indigo07_materials', 'dis_tex',disable_mat_texture
 	material.set_attribute 'indigo07_materials', 'temp',temp
	material.set_attribute 'indigo07_materials', 'no_smooth',disable_smoothing
	material.set_attribute 'indigo07_materials', 'exp_map',exp_map
	material.set_attribute 'indigo07_materials', 'expmap_value',expmap_value
	
	
	SU2IN.assign_properties_to_material

end # function end

def SU2IN::mat_query

table=""
model = Sketchup.active_model
materials = model.materials

typ=[]
index=0

materials.each do |m| 
        	
	if m.alpha<1.0
		typ[index]=m.get_attribute 'indigo07_materials','type',"Specular"
		else
		typ[index]=(m.get_attribute 'indigo07_materials','type',"Diffuse")
	end 
  
	table+=m.display_name.slice(0,10).ljust(20)+"\t"+typ[index]+"\n"
	index +=1
    next
end
      
       
    # Now display the results
	
	header="MATERIAL NAME\tTYPE\n"
	UI.messagebox header+table,MB_OK,"Indigo Material Types"

end


#############load an exr####################3

def SU2IN::load_exr

SU2IN.reset_global_variables

model=Sketchup.active_model

UI.messagebox "Environment maps must be located in the same directory as the Indigo XML file."
prompts=["Filename (eg. Sky.exr)","Type","Width","Gain"]
values=[$exr,$exr_type,$exr_width,$exr_gain]
position=%w[Spherical Lat_Long].join("|")
enums=[nil,position,nil,nil]

results=inputbox prompts,values,enums,"Environment Map Properties"
return if not results


$exr=model.set_attribute 'indigo07_settings','exr',results[0]
$exr_type=model.set_attribute 'indigo07_settings','exr_type',results[1]
$exr_width=model.set_attribute 'indigo07_settings','exr_width',results[2].abs
$exr_gain=model.set_attribute 'indigo07_settings','exr_gain',results[3].abs

if ($exr_type=="Spherical") and ($exr_width==0)
	UI.messagebox "Width cannot be zero."
	SU2IN.load_exr
end

end

#############function to automatically modify Inifile.txt

def SU2IN::modify_indigo_inifile

filearray=[]
index=0
path_index=0
modified_line=""
model=Sketchup.active_model


if $indigo_path==""
	$indigo_path=UI.openpanel("Locate Indigo.exe", "" , "indigo.exe")
	model.set_attribute 'indigo07_settings','indigo_path',$indigo_path
	
end


if $indigo_path.slice("indigo.exe")=="indigo.exe"
		
	if FileTest.exist?($indigo_path)
		inifile_path=$indigo_path.split("indigo.exe")[0] + "inifile.txt"
		if FileTest.exist?(inifile_path)
			inifile=File.new(inifile_path,"r")
			#filearray = inifile.readlines
			filearray = File.readlines(inifile_path,"\n")
			inifile.close
		else
			$indigo_path=""
			model.set_attribute 'indigo07_settings','indigo_path',$indigo_path
			UI.messagebox "Inifile.txt not found.  Auto-render aborted."
			return
		end #end if inifile exists
		
		filearray.each do |line| 
			if line[1..13]=="scenefilepath"
				line="\"scenefilepath\"" + " \""+ $g_su2in_export_file +"\"\n"
				modified_line=line
				path_index=index
			end
			index+=1
			next
		end #end do
		
		if path_index
			filearray[path_index]=modified_line
			inifile=File.new(inifile_path,"w")
			
			for l in filearray
				inifile.puts(l)
			end
				
				
			inifile.close
		else
			UI.messagebox "Inifile.txt is corrupt.  Auto-render aborted."
			return
		end #end if scenefile path not found

	else #if indigo.exe does not exist at given path

		UI.messagebox "Indigo.exe could not be found.  Auto-render aborted."
		$indigo_path=""
		model.set_attribute 'indigo07_settings','indigo_path',$indigo_path
		return
	end #end if indigo.exe exists

else #if filename is not indigo.exe

	UI.messagebox "Indigo.exe could not be found.  Auto-render aborted."
	$indigo_path=""
	model.set_attribute 'indigo07_settings','indigo_path',$indigo_path
	return
		
end #end if filename is indigo.exe


end #function end


def SU2IN::quick_export

result=UI.messagebox ("Quick Export? Warning: Only material and camera settings will be exported.", MB_YESNOCANCEL)
return if not result
return if result==2
$settings_only=true if result==6
SU2IN.export

end


#########################################Material Library Functions##################3
def SU2IN::apply_glass(material)
material.set_attribute 'indigo07_materials','type', "Specular"
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.52
material.set_attribute 'indigo07_materials', 'cau',0.0
material.set_attribute 'indigo07_materials', 'precedence', 10
material.set_attribute 'indigo07_materials', 'sss', "No"
return "SPECULAR"
end

def SU2IN::apply_frosted_glass(material)
material.set_attribute 'indigo07_materials','type', "Glossy Transparent"
material.set_attribute 'indigo07_materials', 'exp', 1000
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.52
material.set_attribute 'indigo07_materials', 'cau', 0.0
material.set_attribute 'indigo07_materials', 'precedence', 10
material.set_attribute 'indigo07_materials', 'sss', "No"
return "GLOSSY TRANSPARENT"
end

def SU2IN::apply_mirror(material)
material.set_attribute 'indigo07_materials','type', "Specular"
material.set_attribute 'indigo07_materials', 'ior', 20.0
material.set_attribute 'indigo07_materials', 'cau', 0.0
material.set_attribute 'indigo07_materials', 'precedence', 10
material.set_attribute 'indigo07_materials', 'sss', "No"
return "SPECULAR"
end

def SU2IN::apply_ceramic(material)
material.set_attribute 'indigo07_materials','type', "Phong"
material.set_attribute 'indigo07_materials', 'exp', 5000
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.5
return "PHONG"
end

def SU2IN::apply_dull_plastic(material)
material.set_attribute 'indigo07_materials','type', "Phong"
material.set_attribute 'indigo07_materials', 'exp', 300
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.33
return "PHONG"
end

def SU2IN::apply_shiny_plastic(material)
material.set_attribute 'indigo07_materials','type', "Phong"
material.set_attribute 'indigo07_materials', 'exp', 10000
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.22
return "PHONG"
end

def SU2IN::apply_leather(material)
material.set_attribute 'indigo07_materials','type', "Phong"
material.set_attribute 'indigo07_materials', 'exp', 50
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.58
return "PHONG"
end

def SU2IN::apply_car_paint(material)
material.set_attribute 'indigo07_materials','type', "Phong"
material.set_attribute 'indigo07_materials', 'exp', 30000
material.set_attribute 'indigo07_materials', 'fres', 1.0
material.set_attribute 'indigo07_materials', 'ior', 1.72
return "PHONG"
end

def SU2IN::apply_Al(material)
material.set_attribute 'indigo07_materials','type', "Al"
end

def SU2IN::apply_Cr(material)
material.set_attribute 'indigo07_materials','type', "Cr"
end

def SU2IN::apply_Cu(material)
material.set_attribute 'indigo07_materials','type', "Cu"
end

def SU2IN::apply_Au(material)
material.set_attribute 'indigo07_materials','type', "Au"
end

def SU2IN::apply_Ag(material)
material.set_attribute 'indigo07_materials','type', "Ag"
end

def SU2IN::apply_Ti(material)
material.set_attribute 'indigo07_materials','type', "Ti"
end

def SU2IN::apply_W(material)
material.set_attribute 'indigo07_materials','type', "W"
end

def SU2IN::apply_Li(material)
material.set_attribute 'indigo07_materials','type', "Li"
end

def SU2IN::apply_Ni(material)
material.set_attribute 'indigo07_materials','type', "Ni"
end

def SU2IN::apply_Pt(material)
material.set_attribute 'indigo07_materials','type', "Pt"
end

def SU2IN::apply_curtains(material)
material.set_attribute 'indigo07_materials','type', "Specular"
material.set_attribute 'indigo07_materials', 'ior', 1.05
material.set_attribute 'indigo07_materials', 'cau', 0.0
material.set_attribute 'indigo07_materials', 'precedence', 10
material.set_attribute 'indigo07_materials', 'sss', "Yes"
material.set_attribute 'indigo07_materials', 'sssval', 3000
mat.alpha=0.75
return "SPECULAR"
end

def SU2IN::apply_lampshade(material)
material.set_attribute 'indigo07_materials','type', "Specular"
material.set_attribute 'indigo07_materials', 'ior', 1.05
material.set_attribute 'indigo07_materials', 'cau', 0.0
material.set_attribute 'indigo07_materials', 'precedence', 10
material.set_attribute 'indigo07_materials', 'sss', "Yes"
material.set_attribute 'indigo07_materials', 'sssval', 5000
return "SPECULAR"
end

def SU2IN::apply_100W(material)
material.set_attribute 'indigo07_materials','type', "Blackbody Light"
material.set_attribute 'indigo07_materials', 'power',100.0
material.set_attribute 'indigo07_materials', 'ole' , 17.5
material.set_attribute 'indigo07_materials', 'temp', 2800
return "BLACKBODY LIGHT"
end

def SU2IN::apply_60W(material)
material.set_attribute 'indigo07_materials','type', "Blackbody Light"
material.set_attribute 'indigo07_materials', 'power', 60.0
material.set_attribute 'indigo07_materials', 'ole' , 14.0
material.set_attribute 'indigo07_materials', 'temp', 2800
return "BLACKBODY LIGHT"
end

def SU2IN::apply_40W(material)
material.set_attribute 'indigo07_materials','type', "Blackbody Light"
material.set_attribute 'indigo07_materials', 'power',40.0
material.set_attribute 'indigo07_materials', 'ole' , 12.6
material.set_attribute 'indigo07_materials', 'temp', 2800
return "BLACKBODY LIGHT"
end

def SU2IN::apply_5W(material)
material.set_attribute 'indigo07_materials','type', "Blackbody Light"
material.set_attribute 'indigo07_materials', 'power',5.0
material.set_attribute 'indigo07_materials', 'ole' , 5.0
material.set_attribute 'indigo07_materials', 'temp', 2800
return "BLACKBODY LIGHT"
end

end #Class end



#---------------------------------------Custom Tool to select Focal distance----------------------
class FocusTool


def initialize
    @ip1 = nil
        
end


def activate
    
    @ip1 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
      
    Sketchup::set_status_text ("Enter 0 for Autofocus"), SB_VCB_LABEL
    
    self.reset(nil)
end


def deactivate(view)
    view.invalidate 
end


def onMouseMove(flags, x, y, view)
    
        
	@ip.pick view, x, y
        if( @ip != @ip1 )
           
            view.invalidate if( @ip.display? or @ip1.display? )
            @ip1.copy! @ip
                        
            view.tooltip = @ip1.tooltip
        end

end


def onLButtonDown(flags, x, y, view)
    
    
	
        @ip1.pick view, x, y
        if( @ip1.valid? )

            	model=Sketchup.active_model		
		$focal_point=model.set_attribute 'indigo07_settings','fp',@ip1.position
		UI.messagebox ("New Focal Point set to #{$focal_point}")
		                    
        end
                
    
    view.lock_inference
	Sketchup.active_model.select_tool nil
end


def onUserText(text, view)
       
    begin
        value = text.to_l
    rescue
       
        UI.beep
        puts "Cannot convert #{text} to a Length"
        value = nil
        Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value

	$focus_distance=value
	if value==0
		UI.messagebox ("Camera set to Autofocus")
		model=Sketchup.active_model
		$focal_point=model.set_attribute 'indigo07_settings','fp',"auto"
	else
    	UI.messagebox ("New Focus Distance set to #{$focus_distance}")
	end

           
    self.reset(view)
	Sketchup.active_model.select_tool nil
end


def draw(view)
    if( @ip1.valid? )
        if( @ip1.display? )
            @ip1.draw(view)
            @drawn = true
        end
        
    end
end



def onCancel(flag, view)
    self.reset(view)
end




# Reset the tool back to its initial state
def reset(view)
       
    Sketchup::set_status_text("Select Point on Focal Plane or Enter Focus Distance in VCB", SB_PROMPT)
    
    
    @ip1.clear
    
    
    if( view )
        view.tooltip = nil
        view.invalidate 
    end
    
   
end


end # class FocusTool

def focustool
    Sketchup.active_model.select_tool FocusTool.new
end


##########################################Context menus##################################
def selected_material
	ent = Sketchup.active_model.selection.first
	return nil if ent==nil
	$selected_material=ent.material if ent!=nil
	return nil if not $selected_material
	return $selected_material
end

UI.add_context_menu_handler do |menu|
	mat=selected_material
        if mat
		
            menu.add_separator
		type=(mat.get_attribute 'indigo07_materials','type',"Diffuse").upcase
            preset_menu=menu.add_submenu("SkIndigo Preset \"#{mat.name}\" (#{type})")
		preset_menu.add_item("Edit Material") {(SU2IN.assign_properties_to_material)}
		preset_menu.add_item("Glass") { (SU2IN.apply_glass(mat)) }
		preset_menu.add_item("Frosted Glass") { (SU2IN.apply_frosted_glass(mat)) }
		preset_menu.add_item("Mirror") { (SU2IN.apply_mirror(mat)) }
		preset_menu.add_item("Ceramic") { (SU2IN.apply_ceramic(mat)) }
		preset_menu.add_item("Dull Plastic") { (SU2IN.apply_dull_plastic(mat)) }
		preset_menu.add_item("Shiny Plastic") { (SU2IN.apply_shiny_plastic(mat)) }
		preset_menu.add_item("Leather") { (SU2IN.apply_leather(mat)) }
		preset_menu.add_item("Car Paint") { (SU2IN.apply_car_paint(mat)) }
		preset_menu.add_item("Aluminum") { (SU2IN.apply_Al(mat)) }
		preset_menu.add_item("Chrome") { (SU2IN.apply_Cr(mat)) }
		preset_menu.add_item("Copper") { (SU2IN.apply_Cu(mat)) }
		preset_menu.add_item("Gold") { (SU2IN.apply_Au(mat)) }
		preset_menu.add_item("Silver") { (SU2IN.apply_Ag(mat)) }
		preset_menu.add_item("Titanium") { (SU2IN.apply_Ti(mat)) }
		preset_menu.add_item("Tungsten") { (SU2IN.apply_W(mat)) }
		preset_menu.add_item("Lithium") { (SU2IN.apply_Li(mat)) }
		preset_menu.add_item("Nickel") { (SU2IN.apply_Ni(mat)) }
		preset_menu.add_item("Platinum") { (SU2IN.apply_Pt(mat)) }
		preset_menu.add_item("Curtains") { (SU2IN.apply_curtains(mat)) }
		preset_menu.add_item("Lampshade") { (SU2IN.apply_lampshade(mat)) }
		preset_menu.add_item("100W Incandescent") { (SU2IN.apply_100W(mat)) }
		preset_menu.add_item("60W Incandescent") { (SU2IN.apply_60W(mat)) }
		preset_menu.add_item("40W Incandescent") { (SU2IN.apply_40W(mat)) }
		preset_menu.add_item("5W Incandescent") { (SU2IN.apply_5W(mat)) }
		
        end
   
end



UI.add_context_menu_handler do |menu|
	
        if $ss_exported and ($g_su2in_model_name!="")
		menu.add_separator if not selected_material	
            menu.add_item("SkIndigo Quick Export") {(SU2IN.quick_export)}
		
        end
   
end


####################Autorun Batch File#########################

def SU2IN::create_batch_file(path)

batch_file_path=path[0..path.length-11]+"autorender.bat"
batch=File.new(batch_file_path,"w")

batch.puts "start \/low indigo.exe"  #runs indigo in low priority
batch.close
return batch_file_path

end




# ---------------------------------------- Menu item
if( not file_loaded?("SkIndigo.rb") )

	main_menu = UI.menu("Plugins").add_submenu("SkIndigo 0.7test5")
	main_menu.add_item("Export Model") { (SU2IN.export) }
	main_menu.add_item("Material Editor") { (SU2IN.assign_properties_to_material) }
	main_menu.add_item("Query Material Types") { (SU2IN.mat_query) }
	main_menu.add_item("Render Settings") { (SU2IN.render_settings)}
	main_menu.add_item("Set Camera Focus") {(focustool)}
	main_menu.add_item("Set Environment Map") {(SU2IN.load_exr)}
	main_menu.add_separator
	main_menu.add_item("About SkIndigo 0.7test5") { (SU2IN.about) }
		
	
end 

#-----------------------------------------------------------
file_loaded("SkIndigo_07.rb")
